===========
PlenoptiCam
===========
.. A light field photography application (macOS_ or Win_) for computational change of perspective view and synthetic focus based on a Standard Plenoptic Camera (SPC).

Description
===========

*PlenoptiCam* comes as a cross-platform, open-source software (see `releases page <releases_>`_) for scientific light field computation with few dependencies and a lean graphical user interface.
Raw exposures from a plenoptic camera require four-dimensional image alignment in order to be decoded as a light field. Provided software addresses this by its ability to calibrate an image taken by a plenoptic camera and extract sub-aperture images or synthetically focused photographs.
This application is meant for researchers, developers, beginners and other fiddlers who like to experiment with light field technology. Its scope comprises custom-types of plenoptic cameras and is thus not limited to Lytro's image data.

|release| |license| |coverage| |downloads| |pypi| |paper|

|actions| 

|colab|

Results
=======

Datesets
--------

.. list-table::
   :widths: 8 8
   :header-rows: 1
   :stub-columns: 0

   * - |OpEx|_
     - |INRIA|_
   * - |opex_demo|
     - |inria_demo|
   * - Custom-built (png, jpg, bmp files)
     - Lytro Illum (lfp, lfr, tar files)
   * - |binder|
     - |colab|

Color equalization
------------------

.. list-table::
   :widths: 4 4 4
   :header-rows: 1
   :stub-columns: 0

   * - Before
     - Target
     - After
   * - |src_lfp|
     - |ref_lfp|
     - |res_lfp|

Depth
-----
.. list-table::
   :widths: 8 8
   :header-rows: 1
   :stub-columns: 0

   * - 2-D depth map
     - 3-D point cloud
   * - |depth_demo|
     - |depth_anim|
   * - pfm file format
     - ply file format

Installation
============

* executable:
    1. download a bundled app found at the `releases page <releases_>`_
    2. run installer / extract archive
    3. work around |WinExeDoc|_ or |MacAppDoc|_ issue
        - macOS Big Sur requires to set ``$ xattr -c /Applications/plenopticam.app`` (|BigSurDoc|_)
    4. run executable (may take a while on first start-up)
    5. user interface will show up (see below)

|

* via pip:
    1. install with ``$ python3 -m pip install plenopticam``
    2. type ``$ plenopticam -g`` to the command line once installation finished

|

* from source:
    1. install Python from https://www.python.org/
    2. download the source_ using ``$ git clone https://github.com/hahnec/plenopticam.git``
    3. go to the root directory ``$ cd plenopticam``
    4. load other packages ``$ python3 -m pip install -r requirements.txt``
    5. install with ``$ sudo python3 setup.py install`` from root directory
    6. if installation ran smoothly, enter ``$ sudo plenopticam -g`` to the command line

Usage
=====

Application
-----------

|gui|

.. |gui| raw:: html

    <p align="center">
        <img src="https://raw.githubusercontent.com/hahnec/plenopticam/master/docs/img/gui_linux.png" width="66%">
    <p>

.. note::
    "Pick folder" checkbox is useful to point to a directory containing calibration archive files of Lytro Illum cameras.
    In such case, the white image calibration file corresponding to the Lytro Illum image is found automatically.

|

0. Download a dataset if you don't call a plenoptic camera your own, e.g.:
    - |OpEx|_
    - |INRIA|_

|

1. Choose your light field photograph using upper **Browse** button. Supported file types are:
    - *bmp*, *jpg* or *png* file from custom-built plenoptic camera
    - *lfr*, *lfp* and *raw* files from Lytro Illum

|

2. Choose calibration source data using lower **Browse** button. Supported file types are:
    - *bmp*, *jpg* or *png* file from custom-built plenoptic camera
    - *caldata-XXX.tar* archive or respective *raw* file from Lytro Illum
    - tick "Pick folder" and point to a directory containing all *tar* files or extracted folders (calibration file will be found automatically)

.. note::
    If you are `using your own Lytro Illum <https://hahnec.github.io/plenopticam/build/html/guide.html#using-your-own-lytro-illum>`__ camera, you first need to extract camera calibration data as a *tar*-archive. To do so, please follow the `instructions guide <https://hahnec.github.io/plenopticam/build/html/guide.html>`__.

|

3. Adjust configuration using **Settings** button:
    - *Micro image patch size*: default is estimated maximum
    - *Refocusing range*: range for shift and sum synthesis
    - *Force re-calibration*: option to re-run calibration
    - *Automatic white balance*: option for white balancing
    - *Refocus refinement*: option for sub-pixel precise refocusing
    - *Scheimpflug*: option to mimic tilted sensor

|

4. Press **Process** to start the computation.

|

Command Line
------------

For computing a stack of light-field images, it may be convenient to iterate through them. This can be done by:

``$ sudo plenopticam -f '/Users/../Folder/' -c 'Users/../caldata-B5144000XXX.tar'``

with necessary write privileges on Unix Systems. A Windows-equivalent command would be as follows:

``plenopticam --file="C:\\..\\Folder\\" --cali="C:\\..\\caldata-B5144000XXX.tar"``

More information on optional arguments can be found using the help parameter

``plenopticam -h``

Results can be found inside a folder named after your light field file which is located in the same directory path.
Depending on provided files and settings, your results folder may contain the following data:

    - sub-aperture images (see *viewpoints_xxx* folder) and view animation given as gif
    - refocused images (see *refo_xxx* folder) and refocus animation given as gif
    - depth map given as pfm and ply file
    - raw image file given as tiff
    - aligned light field as pkl (pickle) file
    - light field metadata as json file (for Lytro files only)
    - scheimpflug focus files

API usage
---------

Usage of PlenoptiCam modules is demonstrated in the following Jupyter notebooks:

`1. Calibration Demo <https://github.com/hahnec/plenopticam/blob/master/examples/01_calib_demo.ipynb>`__

`2. Alignment Demo <https://github.com/hahnec/plenopticam/blob/master/examples/02_align_demo.ipynb>`__

`3. Extraction Demo <https://github.com/hahnec/plenopticam/blob/master/examples/03_xtract_demo.ipynb>`__

`4. Lytro Illum Demo <https://github.com/hahnec/plenopticam/blob/master/examples/04_illum_demo.ipynb>`__

Citation
========

.. code-block:: BibTeX

    @ARTICLE{plenopticam,
        author={Hahne, Christopher and Aggoun, Amar},
        journal={IEEE Transactions on Image Processing},
        title={PlenoptiCam v1.0: A Light-Field Imaging Framework},
        year={2021},
        volume={30},
        number={},
        pages={6757-6771},
        doi={10.1109/TIP.2021.3095671}
    }

Further information
===================

* check out PlenoptiCam's partner project PlenoptiSign_ capable of estimating metric light field geometries
* visit `plenoptic.info <http://www.plenoptic.info>`__ for technical details, animated figures and theoretical backgrounds

.. Image substitutions

.. |release| image:: https://img.shields.io/github/release/hahnec/plenopticam.svg?style=flat-square
    :target: https://github.com/hahnec/plenopticam/releases/
    :alt: release

.. |license| image:: https://img.shields.io/badge/License-GPL%20v3.0-orange.svg?style=flat-square
    :target: https://www.gnu.org/licenses/gpl-3.0.en.html
    :alt: License

.. |code| image:: https://img.shields.io/github/languages/code-size/hahnec/plenopticam.svg?style=flat-square
    :target: https://github.com/hahnec/plenopticam/archive/master.zip
    :alt: Code size

.. |repo| image:: https://img.shields.io/github/repo-size/hahnec/plenopticam.svg?style=flat-square
    :target: https://github.com/hahnec/plenopticam/archive/master.zip
    :alt: Repo size

.. |downloads| image:: https://img.shields.io/github/downloads/hahnec/plenopticam/total?label=Release%20downloads&style=flat-square
    :target: https://github.com/hahnec/plenopticam/releases/
    :alt: Release Downloads

.. |travis| image:: https://img.shields.io/travis/com/hahnec/plenopticam/master?style=flat-square
    :target: https://travis-ci.com/github/hahnec/plenopticam

.. |actions| image:: https://github.com/hahnec/plenopticam/workflows/PlenoptiCam's%20CI%20Pipeline/badge.svg?event=push&style=flat-square
    :target: https://github.com/hahnec/plenopticam/actions

.. |coverage| image:: https://img.shields.io/coveralls/github/hahnec/plenopticam?style=flat-square
    :target: https://coveralls.io/github/hahnec/plenopticam

.. |pypi| image:: https://img.shields.io/pypi/dm/plenopticam?label=PyPI%20downloads&style=flat-square
    :target: https://pypi.org/project/plenopticam/
    :alt: PyPI Downloads

.. |binder| image:: https://img.shields.io/badge/launch-binder-E66581.svg?style=flat-square&logo=data:image/png;base64, 
    :target: https://mybinder.org/v2/gh/hahnec/plenopticam/master?urlpath=lab
    :width: 175

.. |colab| image:: https://colab.research.google.com/assets/colab-badge.svg?style=flat-square
    :target: https://colab.research.google.com/github/hahnec/plenopticam/blob/master/examples/04_illum_demo.ipynb
    :width: 175

.. |paper| image:: http://img.shields.io/badge/paper-arxiv.2010.11687-red.svg?style=flat-square
    :target: https://arxiv.org/pdf/2010.11687.pdf
    :alt: arXiv link

.. |depth_anim| raw:: html

    <img src="https://raw.githubusercontent.com/hahnec/plenopticam/master/docs/img/depth_anim.gif">

.. |depth_demo| raw:: html

    <img src="https://raw.githubusercontent.com/hahnec/plenopticam/master/docs/img/depth_demo_lores.png" max-width:"100%">

.. |pflug_demo| raw:: html

    <img src="https://raw.githubusercontent.com/hahnec/plenopticam/master/docs/img/pflug_demo_lores.png" max-width:"100%">

.. |views_demo| raw:: html

    <img src="https://raw.githubusercontent.com/hahnec/plenopticam/master/docs/img/views_demo_lores.gif" max-width:"100%">

.. |refoc_demo| raw:: html

    <img src="https://raw.githubusercontent.com/hahnec/plenopticam/master/docs/img/refoc_demo_lores.gif" max-width:"100%">

.. |src_lfp| raw:: html

    <img src="https://raw.githubusercontent.com/hahnec/color-matcher/master/tests/data/view_animation_7px.gif" max-height="200px">

.. |ref_lfp| raw:: html

    <img src="https://raw.githubusercontent.com/hahnec/color-matcher/master/tests/data/bee_2.png" max-height="200px">

.. |res_lfp| raw:: html

    <img src="https://raw.githubusercontent.com/hahnec/color-matcher/master/tests/data/view_animation_7px_hm-mkl-hm.gif" max-height="200px">

.. |opex_demo| raw:: html

    <img src="https://raw.githubusercontent.com/hahnec/plenopticam/master/docs/img/demo_custom.gif" height="187px" max-width:"100%">

.. |inria_demo| raw:: html

    <img src="https://raw.githubusercontent.com/hahnec/plenopticam/master/docs/img/demo_illum.gif" height="187px" max-width:"100%">

.. |UoB| raw:: html

    <img src="https://3tkh0x1zl0mb1ta92c2mrvv2-wpengine.netdna-ssl.com/wp-content/uploads/2015/12/LO_KukriGB_Universities_Bedfordshire.png" width="70px">

.. |EUFramework| raw:: html

    <img src="http://www.gsa.europa.eu/sites/default/files/Seventh_Framework_Programme_logo.png" width="100px">

.. |Hahne| raw:: html

    <img src="http://www.christopherhahne.de/images/about_alt.jpg" width="15%">

.. |br| raw:: html

    <br />

.. Hyperlink aliases

.. _source: https://github.com/hahnec/plenopticam/archive/master.zip
.. _macOS: https://github.com/hahnec/plenopticam/releases/download/v0.9.3-beta/plenopticam_0.9.3.dmg
.. _Win: https://github.com/hahnec/plenopticam/releases/download/v0.9.3-beta/plenopticam_0.9.3.msi
.. _Linux: https://github.com/hahnec/plenopticam/releases/download/v0.9.3-beta/plenopticam_0.9.3.tar.gz
.. _PlenoptiSign: https://github.com/hahnec/plenoptisign/
.. _releases: https://github.com/hahnec/plenopticam/releases/

.. |WinExeDoc| replace:: **Windows protection**
.. _WinExeDoc: https://stackoverflow.com/a/65488937

.. |MacAppDoc| replace:: **macOS unidentified developer**
.. _MacAppDoc: https://osxdaily.com/2015/05/04/disable-gatekeeper-command-line-mac-osx/

.. |BigSurDoc| replace:: **see here**
.. _BigSurDoc: https://stackoverflow.com/a/67975297

.. |OpEx| replace:: **OpEx dataset**
.. _OpEx: https://ndownloader.figshare.com/files/5201452

.. |INRIA| replace:: **INRIA dataset**
.. _INRIA: https://www.irisa.fr/temics/demos/IllumDatasetLF/index.html

.. |Stanford| replace:: **Stanford dataset**
.. _Stanford: http://lightfields.stanford.edu/mvlf/

.. |IllumTar| replace:: *using your own Illum data*
.. _IllumTar: https://raw.githubusercontent.com/hahnec/plenopticam/master/docs/build/html/guide.html